/* $Id: TestRunner.java,v 1.6 2004/06/20 15:23:40 fukasawa Exp $ */

/*
 * Original source is JUnit's TestRunner.java
 */

/**
 *  @file    TestRunner.java
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.jyugem.jsunit.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import junit.framework.*;
import junit.runner.*;
import javax.swing.filechooser.*;    // jsunit
import java.lang.reflect.*;          // jsunit
import com.jyugem.secs.*;            // jsunit
import com.jyugem.jsunit.tfw.*;      // jsunit

/**
 * A Swing based user interface to run tests.
 * Enter the name of a class which either provides a static
 * suite method or is a subclass of TestCase.
 * <pre>
 * Synopsis: java junit.swingui.TestRunner [-noloading] [TestCase]
 * </pre>
 * TestRunner takes as an optional argument the name of the testcase class to be run.
 */
public class TestRunner extends BaseTestRunner implements TestRunContext {
    private static final int GAP= 4;
    private static final int HISTORY_LENGTH= 5;

    protected JFrame fFrame;
    private Thread fRunner;
    private TestResult fTestResult;

    private JComboBox fSuiteCombo;
    private ProgressBar fProgressIndicator;
    private DefaultListModel fFailures;
    private JLabel fLogo;
    private CounterPanel fCounterPanel;
    private JButton fRun;
    private JButton fQuitButton;
    private JButton fRerunButton;
    private StatusLine fStatusLine;
    private FailureDetailView fFailureView;
    private JTabbedPane fTestViewTab;
    private JCheckBox fUseLoadingRunner;
    private Vector fTestRunViews= new Vector(); // view associated with tab in tabbed pane

    private static final String TESTCOLLECTOR_KEY= "TestCollectorClass";
    private static final String FAILUREDETAILVIEW_KEY= "FailureViewClass";

    static SECSTestManager m_manager = null;           // jsunit
    Test    m_testSuite = null;                        // jsunit
    String  m_suiteName = null;                        // jsunit
    String  m_fileName = null;                         // jsunit
    private static boolean m_first = true;             // jsunit
    private static final boolean m_isJyusecs = true;   // jsunit
    JFileChooser m_chooser;                            // jsunit
    SECSFileFilter m_xmlfilter;                        // jsunit

    public TestRunner() {
    }

    public static void main(String[] args) {
        m_manager = SECSTestManager.getTestManager();  // jsunit
        new TestRunner().start(args);
    }

    public static void run(Class test) {
        String args[]= { test.getName() };
        main(args);
    }

    public static SECSTestManager getManager() { return m_manager; }      // jsunit

    public void testFailed(final int status, final Test test, final Throwable t) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    switch (status) {
                        case TestRunListener.STATUS_ERROR:
                            fCounterPanel.setErrorValue(fTestResult.errorCount());
                            appendFailure(test, t);
                            break;
                        case TestRunListener.STATUS_FAILURE:
                            fCounterPanel.setFailureValue(fTestResult.failureCount());
                            appendFailure(test, t);
                            break;
                    }
                }
            }
        );
    }

    public void testStarted(String testName) {
        postInfo("Running: "+testName);
    }

    // jsunit : Override BaseTestRunner  >>>
    public synchronized void startTest(Test test) {
        testStarted(test.toString());

        // jsunit : append below
        Class jyuclazz = null;
        if (test.getClass().getName().equals("com.jyugem.jsunit.tfw.ActiveTestCase")) {
            jyuclazz = ActiveTestCase.class;
        } else if (test.getClass().getName().equals("com.jyugem.jsunit.tfw.PassiveTestCase")) {
            jyuclazz = PassiveTestCase.class;
        }
        if (jyuclazz != null) {
            try {
                Class parmClass[] = new Class[] { SECSTestManager.class };
                Method getmethod = jyuclazz.getMethod("setManager", parmClass);
                Object parm[] = new Object[] { m_manager };
                getmethod.invoke(test, parm);
            } catch (Exception e) {
                showStatus("Can't set manager.");
            }
        }
    }
    // jsunit <<<


    public void testEnded(String stringName) {
        synchUI();
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    if (fTestResult != null) {
                        fCounterPanel.setRunValue(fTestResult.runCount());
                        fProgressIndicator.step(fTestResult.runCount(), fTestResult.wasSuccessful());
                    }
                }
            }
        );
    }

    public void setSuite(String suiteName) {
        fSuiteCombo.getEditor().setItem(suiteName);
    }

    private void addToHistory(final String suite) {
        for (int i= 0; i < fSuiteCombo.getItemCount(); i++) {
            if (suite.equals(fSuiteCombo.getItemAt(i))) {
                fSuiteCombo.removeItemAt(i);
                fSuiteCombo.insertItemAt(suite, 0);
                fSuiteCombo.setSelectedIndex(0);
                return;
            }
        }
        fSuiteCombo.insertItemAt(suite, 0);
        fSuiteCombo.setSelectedIndex(0);
        pruneHistory();
    }

    private void pruneHistory() {
        int historyLength= getPreference("maxhistory", HISTORY_LENGTH);
        if (historyLength < 1)
            historyLength= 1;
        for (int i= fSuiteCombo.getItemCount()-1; i > historyLength-1; i--)
            fSuiteCombo.removeItemAt(i);
    }

    private void appendFailure(Test test, Throwable t) {
        fFailures.addElement(new TestFailure(test, t));
        if (fFailures.size() == 1)
            revealFailure(test);
    }

    private void revealFailure(Test test) {
        for (Enumeration e= fTestRunViews.elements(); e.hasMoreElements(); ) {
            TestRunView v= (TestRunView) e.nextElement();
            v.revealFailure(test);
        }
    }

    protected void aboutToStart(final Test testSuite) {
        for (Enumeration e= fTestRunViews.elements(); e.hasMoreElements(); ) {
            TestRunView v= (TestRunView) e.nextElement();
            v.aboutToStart(testSuite, fTestResult);
        }
    }

    protected void runFinished(final Test testSuite) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    for (Enumeration e= fTestRunViews.elements(); e.hasMoreElements(); ) {
                        TestRunView v= (TestRunView) e.nextElement();
                        v.runFinished(testSuite, fTestResult);
                    }
                }
            }
        );
    }

    protected CounterPanel createCounterPanel() {
        return new CounterPanel();
    }

    protected JPanel createFailedPanel() {
        JPanel failedPanel= new JPanel(new GridLayout(0, 1, 0, 2));
        fRerunButton= new JButton("Run");
        fRerunButton.setEnabled(false);
        fRerunButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    rerun();
                }
            }
        );
        failedPanel.add(fRerunButton);
        return failedPanel;
    }

    protected FailureDetailView createFailureDetailView() {
        String className= BaseTestRunner.getPreference(FAILUREDETAILVIEW_KEY);
        if (className != null) {
            Class viewClass= null;
            try {
                viewClass= Class.forName(className);
                return (FailureDetailView)viewClass.newInstance();
            } catch(Exception e) {
                JOptionPane.showMessageDialog(fFrame, "Could not create Failure DetailView - using default view");
            }
        }
        return new DefaultFailureDetailView();
    }

    /**
     * Creates the JUnit menu. Clients override this
     * method to add additional menu items.
     */
    protected JMenu createJUnitMenu() {
        JMenu menu= new JMenu("JUnit");
        menu.setMnemonic('J');

         JMenuItem mi0= new JMenuItem("Open...");                  // jsunit
        mi0.addActionListener(                                     // jsunit
            new ActionListener() {                                 // jsunit
                public void actionPerformed(ActionEvent event) {   // jsunit
                    openTestFile();                                // jsunit
                }                                                  // jsunit
            }                                                      // jsunit
        );                                                         // jsunit
        mi0.setMnemonic('O');                                      // jsunit
        menu.add(mi0);                                             // jsunit

        JMenuItem mi1= new JMenuItem("About...");
        mi1.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    about();
                }
            }
        );
        mi1.setMnemonic('A');
        menu.add(mi1);

        menu.addSeparator();
        JMenuItem mi2= new JMenuItem(" Exit ");
        mi2.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    terminate();
                }
            }
        );
        mi2.setMnemonic('x');
        menu.add(mi2);

        return menu;
    }

    protected JFrame createFrame() {
        JFrame frame= new JFrame("JUnit for SECS");       // jsunit
        Image icon= loadFrameIcon();
        if (icon != null)
            frame.setIconImage(icon);
        frame.getContentPane().setLayout(new BorderLayout(0, 0));

        frame.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    terminate();
                }
            }
        );
        return frame;
    }

    protected JLabel createLogo() {
        JLabel label;
        Icon icon= getIconResource(BaseTestRunner.class, "logo.gif");
        if (icon != null)
            label= new JLabel(icon);
        else
            label= new JLabel("JV");
        label.setToolTipText("JUnit Version "+Version.id());
        return label;
    }

    protected void createMenus(JMenuBar mb) {
        mb.add(createJUnitMenu());
    }

    protected JCheckBox createUseLoaderCheckBox() {
        boolean useLoader= useReloadingTestSuiteLoader();
        JCheckBox box= new JCheckBox("Reload classes every run", useLoader);
        box.setToolTipText("Use a custom class loader to reload the classes for every run");
        if (inVAJava())
            box.setVisible(false);
        return box;
    }

    protected JButton createQuitButton() {
         // spaces required to avoid layout flicker
         // Exit is shorter than Stop that shows in the same column
        JButton quit= new JButton(" Exit ");
        quit.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    terminate();
                }
            }
        );
        return quit;
    }

    protected JButton createRunButton() {
        JButton run= new JButton("Run");
        run.setEnabled(true);
        run.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    runSuite();
                }
            }
        );
        return run;
    }

    protected Component createBrowseButton() {
        JButton browse= new JButton("...");
        browse.setToolTipText("Select a Test class");
        browse.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    browseTestClasses();
                }
            }
        );
        return browse;
    }

        protected Component createReloadButton() {             // jsunit
        JButton reload= new JButton("Reload");                 // jsunit
        reload.setToolTipText("Reload a Test class");          // jsunit
        reload.addActionListener(                              // jsunit
            new ActionListener() {                             // jsunit
                public void actionPerformed(ActionEvent e) {   // jsunit
                    reloadTestClasses();                       // jsunit
                }                                              // jsunit
            }                                                  // jsunit
        );                                                     // jsunit
        return reload;                                         // jsunit
    }                                                          // jsunit

    protected StatusLine createStatusLine() {
        return new StatusLine(380);
    }

    protected JComboBox createSuiteCombo() {
        JComboBox combo= new JComboBox();
        combo.setEditable(true);
        combo.setLightWeightPopupEnabled(false);

        combo.getEditor().getEditorComponent().addKeyListener(
            new KeyAdapter() {
                public void keyTyped(KeyEvent e) {
                    textChanged();
                    if (e.getKeyChar() == KeyEvent.VK_ENTER)
                        runSuite();
                }
            }
        );
        try {
            loadHistory(combo);
        } catch (IOException e) {
            // fails the first time
        }
        combo.addItemListener(
            new ItemListener() {
                public void itemStateChanged(ItemEvent event) {
                    if (event.getStateChange() == ItemEvent.SELECTED) {
                        textChanged();
                    }
                }
            }
        );
        return combo;
    }

    protected JTabbedPane createTestRunViews() {
        JTabbedPane pane= new JTabbedPane(JTabbedPane.BOTTOM);

        FailureRunView lv= new FailureRunView(this);
        fTestRunViews.addElement(lv);
        lv.addTab(pane);

        TestHierarchyRunView tv= new TestHierarchyRunView(this);
        fTestRunViews.addElement(tv);
        tv.addTab(pane);

        pane.addChangeListener(
            new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    testViewChanged();
                }
            }
        );
        return pane;
    }

    public void testViewChanged() {
        TestRunView view= (TestRunView)fTestRunViews.elementAt(fTestViewTab.getSelectedIndex());
        view.activate();
    }

    protected TestResult createTestResult() {
        return new TestResult();
    }

    protected void createFileChooser() {               // jsunit
        m_chooser = new JFileChooser();                // jsunit
        m_xmlfilter = new SECSFileFilter();            // jsunit
        m_chooser.addChoosableFileFilter(m_xmlfilter); // jsunit
    }                                                  // jsunit

    protected JFrame createUI(String suiteName) {
        JFrame frame= createFrame();
        JMenuBar mb= new JMenuBar();
        createMenus(mb);
        frame.setJMenuBar(mb);
        createFileChooser();                           // jsunit

        JLabel suiteLabel= new JLabel("Test class name:");
        fSuiteCombo= createSuiteCombo();
        fRun= createRunButton();
        frame.getRootPane().setDefaultButton(fRun);
        Component browseButton= createBrowseButton();
        Component reloadButton= createReloadButton();      // jsunit

        fUseLoadingRunner= createUseLoaderCheckBox();
        fProgressIndicator= new ProgressBar();
        fCounterPanel= createCounterPanel();

        fFailures= new DefaultListModel();

        fTestViewTab= createTestRunViews();
        JPanel failedPanel= createFailedPanel();

        fFailureView= createFailureDetailView();
        JScrollPane tracePane= new JScrollPane(fFailureView.getComponent(), JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

        fStatusLine= createStatusLine();
        fQuitButton= createQuitButton();
        fLogo= createLogo();

        JPanel panel= new JPanel(new GridBagLayout());

        addGrid(panel, suiteLabel,  0, 0, 2, GridBagConstraints.HORIZONTAL,     1.0, GridBagConstraints.WEST);
        addGrid(panel, reloadButton, 2, 0, 1, GridBagConstraints.NONE,       0.0, GridBagConstraints.CENTER);    // jsunit
        addGrid(panel, fSuiteCombo,     0, 1, 1, GridBagConstraints.HORIZONTAL,     1.0, GridBagConstraints.WEST);
        addGrid(panel, browseButton,    1, 1, 1, GridBagConstraints.NONE,           0.0, GridBagConstraints.WEST);
        addGrid(panel, fRun,        2, 1, 1, GridBagConstraints.HORIZONTAL,     0.0, GridBagConstraints.CENTER);

        addGrid(panel, fUseLoadingRunner,   0, 2, 3, GridBagConstraints.NONE, 1.0, GridBagConstraints.WEST);
        //addGrid(panel, new JSeparator(),  0, 3, 3, GridBagConstraints.HORIZONTAL, 1.0, GridBagConstraints.WEST);


        addGrid(panel, fProgressIndicator,  0, 3, 2, GridBagConstraints.HORIZONTAL,     1.0, GridBagConstraints.WEST);
        addGrid(panel, fLogo,           2, 3, 1, GridBagConstraints.NONE,           0.0, GridBagConstraints.NORTH);

        addGrid(panel, fCounterPanel,    0, 4, 2, GridBagConstraints.NONE,          0.0, GridBagConstraints.WEST);
        addGrid(panel, new JSeparator(),    0, 5, 2, GridBagConstraints.HORIZONTAL, 1.0, GridBagConstraints.WEST);
        addGrid(panel, new JLabel("Results:"),  0, 6, 2, GridBagConstraints.HORIZONTAL,     1.0, GridBagConstraints.WEST);

        JSplitPane splitter= new JSplitPane(JSplitPane.VERTICAL_SPLIT, fTestViewTab, tracePane);
        addGrid(panel, splitter,     0, 7, 2, GridBagConstraints.BOTH,          1.0, GridBagConstraints.WEST);

        addGrid(panel, failedPanel,      2, 7, 1, GridBagConstraints.HORIZONTAL,    0.0, GridBagConstraints.NORTH/*CENTER*/);

        addGrid(panel, fStatusLine,      0, 9, 2, GridBagConstraints.HORIZONTAL,    1.0, GridBagConstraints.CENTER);
        addGrid(panel, fQuitButton,      2, 9, 1, GridBagConstraints.HORIZONTAL,    0.0, GridBagConstraints.CENTER);

        frame.setContentPane(panel);
        frame.pack();
        frame.setLocation(200, 200);
        return frame;
    }

    private void addGrid(JPanel p, Component co, int x, int y, int w, int fill, double wx, int anchor) {
        GridBagConstraints c= new GridBagConstraints();
        c.gridx= x; c.gridy= y;
        c.gridwidth= w;
        c.anchor= anchor;
        c.weightx= wx;
        c.fill= fill;
        if (fill == GridBagConstraints.BOTH || fill == GridBagConstraints.VERTICAL)
            c.weighty= 1.0;
        c.insets= new Insets(y == 0 ? 10 : 0, x == 0 ? 10 : GAP, GAP, GAP);
        p.add(co, c);
    }

    protected String getSuiteText() {
        if (fSuiteCombo == null)
            return "";
        return (String)fSuiteCombo.getEditor().getItem();
    }

    public ListModel getFailures() {
        return fFailures;
    }

    public void insertUpdate(DocumentEvent event) {
        textChanged();
    }

    public void browseTestClasses() {
        TestCollector collector= createTestCollector();
        TestSelector selector= new TestSelector(fFrame, collector);
        if (selector.isEmpty()) {
            JOptionPane.showMessageDialog(fFrame, "No Test Cases found.\nCheck that the configured \'TestCollector\' is supported on this platform.");
            return;
        }
        selector.show();
        String className= selector.getSelectedItem();
        // if (className != null)      // jsunit
        //  setSuite(className);       // jsunit
        if (className != null) {       // jsunit
            setSuite(className);       // jsunit
            loadSuite();               // jsunit
        }                              // jsunit
    }

    public void openTestFile() {                     // jsunit <<<
        if (m_fileName != null) {
            File curFile = new File(m_fileName);
            m_chooser.setSelectedFile(curFile);
            m_chooser.setCurrentDirectory(curFile);
        }
        int retval = m_chooser.showDialog(fFrame, null);
        if (retval == JFileChooser.APPROVE_OPTION) {
            File theFile = m_chooser.getSelectedFile();
            if (theFile != null) {
                if (theFile.isFile()) {
                    String filename = theFile.getName();
                    if (filename != null) {
                        m_manager.setTestFile(filename);
                        m_fileName = filename;
                        loadSuite();
                        System.gc();
                    }
                }
            }
        }
    }                                                 // jsunit >>>

    public void reloadTestClasses() {  // jsunit
        loadSuite();                   // jsunit
        System.gc();                   // jsunit
    }                                  // jsunit

    TestCollector createTestCollector() {
        String className= BaseTestRunner.getPreference(TESTCOLLECTOR_KEY);
        if (className != null) {
            Class collectorClass= null;
            try {
                collectorClass= Class.forName(className);
                return (TestCollector)collectorClass.newInstance();
            } catch(Exception e) {
                JOptionPane.showMessageDialog(fFrame, "Could not create TestCollector - using default collector");
            }
        }
        return new SimpleTestCollector();
    }

    private Image loadFrameIcon() {
        ImageIcon icon= (ImageIcon)getIconResource(BaseTestRunner.class, "smalllogo.gif");
        if (icon != null)
            return icon.getImage();
        return null;
    }

    private void loadHistory(JComboBox combo) throws IOException {
        BufferedReader br= new BufferedReader(new FileReader(getSettingsFile()));
        int itemCount= 0;
        try {
            String line;
            while ((line= br.readLine()) != null) {
                combo.addItem(line);
                itemCount++;
            }
            if (itemCount > 0)
                combo.setSelectedIndex(0);

        } finally {
            br.close();
        }
    }

    private File getSettingsFile() {
        String home= System.getProperty("user.home");
        return new File(home,".junitsession");
    }

    private void postInfo(final String message) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    showInfo(message);
                }
            }
        );
    }

    private void postStatus(final String status) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    showStatus(status);
                }
            }
        );
    }

    public void removeUpdate(DocumentEvent event) {
        textChanged();
    }

    private void rerun() {
        TestRunView view= (TestRunView)fTestRunViews.elementAt(fTestViewTab.getSelectedIndex());
        Test rerunTest= view.getSelectedTest();
        if (rerunTest != null)
            rerunTest(rerunTest);
    }

    private void rerunTest(Test test) {
        Test reloadedTest= null;                                // jsunit
        if (m_isJyusecs) {                                      // jsunit
            reloadedTest = test;                                // jsunit
        } else {                                                // jsunit
            if (!(test instanceof TestCase)) {
                showInfo("Could not reload "+ test.toString());
                return;
            }
            // Test reloadedTest= null;                         // jsunit
            TestCase rerunTest= (TestCase)test;

            try {
                Class reloadedTestClass= getLoader().reload(test.getClass());
                reloadedTest= TestSuite.createTest(reloadedTestClass, rerunTest.getName());
            } catch(Exception e) {
                showInfo("Could not reload "+ test.toString());
                return;
            }
        }                                                           // jsunit
        // TestResult result= new TestResult();                     // jsunit
        // reloadedTest.run(result);                                // jsunit
        final TestResult result= new TestResult();                  // jsunit
        result.addListener(TestRunner.this);                        // jsunit
        final Test retest = reloadedTest;                           // jsunit
        m_manager.responder().setDispatch(true);                    // jsunit
        Thread rerunner = new Thread("RerunTestRunner-Thread") {    // jsunit
            public void run() {                                     // jsunit
                retest.run(result);                                 // jsunit
                m_manager.responder().setDispatch(false);           // jsunit
            }                                                       // jsunit
        };                                                          // jsunit
        rerunner.start();                                           // jsunit

        String message= reloadedTest.toString();
        if(result.wasSuccessful())
            showInfo(message+" was successful");
        else if (result.errorCount() == 1)
            showStatus(message+" had an error");
        else
            showStatus(message+" had a failure");
    }

    protected void reset() {
        fCounterPanel.reset();
        fProgressIndicator.reset();
        fRerunButton.setEnabled(false);
        fFailureView.clear();
        fFailures.clear();
    }

    protected void runFailed(String message) {
        showStatus(message);
        fRun.setText("Run");
        fRunner= null;
    }

    synchronized public void runSuite() {
        if (fRunner != null) {
            fTestResult.stop();
        } else {
            final String suiteName;                         // jsunit
            final Test testSuite;                           // jsunit
            if (m_first) {                                  // jsunit
                setLoading(shouldReload());
                reset();
                showInfo("Load Test Case...");
                suiteName= getSuiteText();                  // jyutest
                testSuite= getTest(suiteName);              // jyutest
            } else {                                        // jyutest
                suiteName= m_suiteName;                     // jyutest
                testSuite= m_testSuite;                     // jyutest
            }                                               // jyutest
            if (testSuite != null) {
                addToHistory(suiteName);
                doRunTest(testSuite);
            }
        }
    }

    public void loadSuite() {                      // jsunit  <<<
        Test testSuite = null;
        setLoading(true);
        reset();
        showInfo("Load Test Case...");
        final String suiteName= getSuiteText();
        testSuite = getTest(suiteName);
        m_testSuite = testSuite;
        m_suiteName = suiteName;
        m_first = false;
        if (m_testSuite != null) {
            addToHistory(m_suiteName);
        }
        fTestResult= createTestResult();
        fTestResult.addListener(TestRunner.this);
        aboutToStart(testSuite);
    }                                              // jsunit  >>>

    private boolean shouldReload() {
        return !inVAJava() && fUseLoadingRunner.isSelected();
    }


    synchronized protected void runTest(final Test testSuite) {
        if (fRunner != null) {
            fTestResult.stop();
        } else {
            reset();
            if (testSuite != null) {
                doRunTest(testSuite);
            }
        }
    }

    private void doRunTest(final Test testSuite) {
        setButtonLabel(fRun, "Stop");

        m_manager.responder().setDispatch(true);            // jsunit

        fRunner= new Thread("TestRunner-Thread") {
            public void run() {
                TestRunner.this.start(testSuite);
                postInfo("Running...");

                long startTime= System.currentTimeMillis();
                testSuite.run(fTestResult);

                if (fTestResult.shouldStop()) {
                    postStatus("Stopped");
                } else {
                    long endTime= System.currentTimeMillis();
                    long runTime= endTime-startTime;
                    postInfo("Finished: " + elapsedTimeAsString(runTime) + " seconds");
                }
                runFinished(testSuite);
                setButtonLabel(fRun, "Run");

                m_manager.responder().setDispatch(false);   // jsunit

                fRunner= null;
                System.gc();
            }
        };
        // make sure that the test result is created before we start the
        // test runner thread so that listeners can register for it.
        fTestResult= createTestResult();
        fTestResult.addListener(TestRunner.this);
        aboutToStart(testSuite);

        fRunner.start();
    }

    private void saveHistory() throws IOException {
        BufferedWriter bw= new BufferedWriter(new FileWriter(getSettingsFile()));
        try {
            for (int i= 0; i < fSuiteCombo.getItemCount(); i++) {
                String testsuite= fSuiteCombo.getItemAt(i).toString();
                bw.write(testsuite, 0, testsuite.length());
                bw.newLine();
            }
        } finally {
            bw.close();
        }
    }

    private void setButtonLabel(final JButton button, final String label) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    button.setText(label);
                }
            }
        );
    }

    public void handleTestSelected(Test test) {
        // fRerunButton.setEnabled(test != null && (test instanceof TestCase)); // jsunit
        fRerunButton.setEnabled(test != null);                                    // jsunit
        showFailureDetail(test);
    }

    private void showFailureDetail(Test test) {
        if (test != null) {
            ListModel failures= getFailures();
            for (int i= 0; i < failures.getSize(); i++) {
                TestFailure failure= (TestFailure)failures.getElementAt(i);
                if (failure.failedTest() == test) {
                    fFailureView.showFailure(failure);
                    return;
                }
            }
        }
        fFailureView.clear();
    }

    private void showInfo(String message) {
        fStatusLine.showInfo(message);
    }

    private void showStatus(String status) {
        fStatusLine.showError(status);
    }

    /**
     * Processes the command line arguments and
     * returns the name of the suite class to run or null
     */
    protected String processArguments(String[] args) {        // jsunit  <<<
        String suiteName= null;
        for (int i= 0; i < args.length; i++) {
            if (args[i].equals("-noloading")) {
                setLoading(false);
            } else if (args[i].equals("-nofilterstack")) {
            //     fgFilterStack= false;
            } else if (args[i].equals("-c")) {
                if (args.length > i+1)
                    suiteName= extractClassName(args[i+1]);
                else
                    System.out.println("Missing Test class name");
                i++;
            } else if (args[i].equals("-x")) {
                if (args.length > i+1)
                    m_fileName = args[i+1];
                else
                    System.out.println("Missing Test file name");
                i++;
            } else {
                suiteName= args[i];
            }
        }
        return suiteName;
    }                                                        // jsunit  >>>

    /**
     * Starts the TestRunner
     */
    public void start(String[] args) {
        String suiteName= processArguments(args);
        fFrame= createUI(suiteName);
        fFrame.pack();
        fFrame.setVisible(true);

        if (suiteName != null) {
            setSuite(suiteName);
            // runSuite();             // jsunit
            loadSuite();               // jsunit
        }
    }

    private void start(final Test test) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    int total= test.countTestCases();
                    fProgressIndicator.start(total);
                    fCounterPanel.setTotal(total);
                }
            }
        );
    }

    /**
     * Wait until all the events are processed in the event thread
     */
    private void synchUI() {
        try {
            SwingUtilities.invokeAndWait(
                new Runnable() {
                    public void run() {}
                }
            );
        }
        catch (Exception e) {
        }
    }

    /**
     * Terminates the TestRunner
     */
    public void terminate() {
        fFrame.dispose();
        try {
            saveHistory();
        } catch (IOException e) {
            System.out.println("Couldn't save test run history");
        }
        System.exit(0);
    }

    public void textChanged() {
        fRun.setEnabled(getSuiteText().length() > 0);
        clearStatus();
    }

    protected void clearStatus() {
        fStatusLine.clear();
    }

    public static Icon getIconResource(Class clazz, String name) {
        URL url= clazz.getResource(name);
        if (url == null) {
            System.err.println("Warning: could not load \""+name+"\" icon");
            return null;
        }
        return new ImageIcon(url);
    }

    private void about() {
        AboutDialog about= new AboutDialog(fFrame);
        about.show();
    }

    /**
     * Returns the Test corresponding to the given suite. This is
     * a template method, subclasses override runFailed(), clearStatus().
     */
    public Test getTest(String suiteClassName) {            // jsunit <<<
        if (suiteClassName.length() <= 0) {
            clearStatus();
            return null;
        }
        Class testClass= null;
        try {
            // testClass= loadSuiteClass(suiteClassName);
            ClassLoader loader = ClassLoader.getSystemClassLoader();
            testClass= loader.loadClass(suiteClassName);
        } catch (ClassNotFoundException e) {
            String clazz= e.getMessage();
            if (clazz == null)
                clazz= suiteClassName;
            runFailed("Class not found \""+clazz+"\"");
            return null;
        } catch(Exception e) {
            runFailed("Error: "+e.toString());
            return null;
        }
        Method suiteMethod= null;
        try {
            Class parms[] = new Class[] { SECSTestManager.class };
            suiteMethod= testClass.getMethod("suite", parms);
        } catch(Exception e) {
            // try to extract a test suite automatically
            clearStatus();
            return new TestSuite(testClass);
        }
        Test test= null;
        try {
            Object[] args = new Object[] {m_manager};
            test= (Test)suiteMethod.invoke(null, args); // static method
            if (test == null)
                return test;
        }
        catch (InvocationTargetException e) {
            runFailed("Failed to invoke suite():" + e.getTargetException().toString());
            return null;
        }
        catch (IllegalAccessException e) {
            runFailed("Failed to invoke suite():" + e.toString());
            return null;
        }

        clearStatus();
        return test;
    }                                                         // jsunit >>>

}