// $Id: JSInt8.cpp,v 1.5 2004/06/20 15:23:40 fukasawa Exp $

//=============================================================================
/**
 *  @file    JSInt8.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL

#include "JSInt8.h"
#include "BS2Array.h"
#include "BS2List.h"

static const char * _className = "java/lang/Long";

void JSInt8::init(JNIEnv * jenv, jclass itemClass)
{
    TRACE_FUNCTION(TRL_LOW, "JSInt8::init");

    initClass(jenv, itemClass);

    m_class = m_jenv->FindClass(_className);
    ASSERT_JCLASS(m_class, _className);

    m_vget = m_jenv->GetMethodID(m_class, "longValue", "()J");
    ASSERT_METHODID(m_vget, "longValue", "()J");

    m_new = m_jenv->GetMethodID(m_class, "<init>", "(J)V");
    ASSERT_METHODID(m_new, "<init>", "(J)V");

    m_item = m_jenv->GetMethodID(itemClass, "<init>",
                                            "(Ljava/lang/String;J)V");
    ASSERT_METHODID(m_item, "<init>", "(Ljava/lang/String;J)V");

    m_array= jenv->GetMethodID(itemClass, "<init>",
                                          "(Ljava/lang/String;[J)V");
    ASSERT_METHODID(m_array, "<init>", "(Ljava/lang/String;[J)V");
}

// secs value to atom object
jobject JSInt8::toObj(BS2Atom * atom)
{
    TRACE_FUNCTION(TRL_LOW, "JSInt8::toObj");
    jobject obj = m_jenv->NewObject(m_class, m_new,
                                    reinterpret_cast<BS2Int8 *>(atom)->value());
    return obj;
}

// secs value to item object
jobject JSInt8::toItemObj(BS2Atom * atom, jstring itemName)
{
    TRACE_FUNCTION(TRL_LOW, "JSInt8::toItemObj");
    jobject obj;
    int format = atom->format();
    if (format & ATOM_ARRAY)
    {
        BS2Int8Array * arrayAtom = reinterpret_cast<BS2Int8Array *>(atom);
        int   len = arrayAtom->count();
        LONGLONG * values = arrayAtom->value();

        jlongArray array =  m_jenv->NewLongArray(len);
        BEEAssert(array != NULL);
        for (int i = 0; i < len; i++)
        {
            jlong val = *(values + i);
            m_jenv->SetLongArrayRegion(array, (jsize)i, 1, &val);
        }
        obj = m_jenv->NewObject(m_itemClass, m_array, itemName, array);
        m_jenv->ReleaseLongArrayElements(array, NULL, JNI_ABORT); // !?
    }
    else
    {
        obj = m_jenv->NewObject(m_itemClass, m_item, itemName,
                                reinterpret_cast<BS2Int8 *>(atom)->value());
    }
    return obj;
}

// jobject to secs value
BS2Atom * JSInt8::toAtom(jobject valObj)
{
    TRACE_FUNCTION(TRL_LOW, "JSInt8::toAtom");

    BS2Atom * atom;
    if (valObj != NULL)
    {
        jlong longval = m_jenv->CallLongMethod(valObj, m_vget);
        atom = new BS2Int8(longval);
    }
    else
    {
        atom = new BS2Int8();
    }
    return atom;
}

// jobject to secs array
BS2Atom * JSInt8::toArray(jobject valObj)
{
    TRACE_FUNCTION(TRL_LOW, "JSInt8::toArray");

    if (valObj == NULL)
    {
        TRACE_ERROR(("Empty member object\n"));
        return NULL;
    }

    jobjectArray array = (jobjectArray)valObj;
    jsize len      = m_jenv->GetArrayLength(array);
    LONGLONG * values = new LONGLONG[len];
    for (int i = 0; i < len; i++)
    {
        jobject obj = m_jenv->GetObjectArrayElement(array, i);
        jlong  val = m_jenv->CallLongMethod(obj, m_vget);
        *(values + i) = (LONGLONG)val;
    }
    BS2Int8Array * atom = new BS2Int8Array();
    atom->add(values, len);
    delete[] values;

    return atom;
}

// jobject to secs list
int JSInt8::toList(jobject valObj, BS2List * listAtom)
{
    TRACE_FUNCTION(TRL_LOW, "JSInt8::toList");

    jint len = m_jenv->CallIntMethod(valObj, m_listSize);
    for (int i = 0; i < len; i++)
    {
        jobject mbrObj = m_jenv->CallObjectMethod(valObj, m_listGet, i);
        jlong val = m_jenv->CallLongMethod(mbrObj, m_vget);
        BS2Atom * atom = new BS2Int8((LONGLONG)val);
        listAtom->add(*atom);
        delete atom;
    }
    return 0;
}

